{% extends "admin/change_form.html" %}
{% load i18n %}

{% block extrahead %}{{ block.super }}
    {% include 'requery/include_ace.html' %}
{% endblock %}


{% block object-tools-items %}
    <li><a href="history/" class="historylink">{% trans "History" %}</a></li>
    {% if has_absolute_url %}<li><a href="{% url 'admin:requery_query_form' original.id %}" class="viewsitelink">{% trans "Running" %}</a></li>{% endif%}
{% endblock %}