<script type="text/javascript" src="{{ STATIC_URL }}requery/ace.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}requery/theme-textmate.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}requery/mode-sql.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}requery/config.js"></script>
<style type="text/css">
    #syntax_text {
        width: 100%;
        height: 400px;
        display: block;
        position: relative;
    }
    #id_text {
        display: none;
    }
</style>